const express = require('express');
const bodyParser = require('body-parser');
const { run } = require('./stepper');
const { getRGDS } = require('./rgds');

const { port, noHtml, apiKey } = require('./conf');

let runData = [];
let realLog = console.log;
let realError = console.error;
let hasErrors = false;
console.log = function () {
  runData.push([...arguments].reduce((reducer, logLine) => `${reducer} ${logLine}`, ''));
  return realLog.apply(console, arguments);
};
console.error = function () {
  hasErrors = true;
  runData.push([...arguments].reduce((reducer, errorLine) => `${reducer} ${errorLine}`, ''));
  return realError.apply(console, arguments);
};


const http = express();

http.use(bodyParser.json());
http.use(bodyParser.urlencoded({ extended: true }));

http.set('view engine', 'pug');
http.set('views', './html');

http.get('/', async (req, res) => {
  if (!noHtml) {
    res.render('index', { rgds: await getRGDS() });
  } else {
    res.status(403);
    res.send('No HTML UI configured');
  }
});

http.post('/',
  (req, res, next) => {
    const key = req.headers.key ? req.headers.key : (req.body.key ? req.body.key : "");
    if (key !== apiKey) {
      if (req.accepts('html')) {
        res.render('error-key');
      } else {
        res.sendStatus(401);
      }
    } else {
      next();
    }
  },
  async (req, res) => {
    console.log(req.accepted);
    const data = Object.keys(req.body).length ? req.body : (req.query || {});
    if (!data.project) {
      res.status(400);
      res.send('Missing project data in request body or query');
    } else {
      try {
        console.log(`Start running project duplication of ${data.project}`);
        runData = [];
        hasErrors = false;
        await run([data.project]);

        if (hasErrors) {
          res.status(500, 'Error during run');
        } else {
          res.status(200, 'Run successful');
        }

        if (req.accepts('html')) {
          runData = runData.map(line => {
            let color = 'is-info';
            if (line.includes('Error') || line.includes('error')) {
              color = 'is-danger';
            } else if (line.includes('Veri')) {
              color = 'is-warning';
            } else if (line.includes('Run') || line.includes('run') || line.includes('Duplicat')) {
              color = 'is-success';
            }
            return {
              line,
              color
            };
          });
          res.render('result', { result: runData });
        } else {
          res.send(runData);
        }
      } catch (err) {
        if (req.accepts('html')) {
          res.render('result', { result: JSON.stringify(err, null, 4) });
        } else {
          res.status(500);
          res.send(`Error during run ${JSON.stringify(err, null, 4)}`);
        }
      }
    }
  });

http.listen(port, () => {
  console.log(`Start DS Project Duplicate Server on port ${port}`);
});
