#!/usr/bin/env node
const program = require('commander');
const { run, configure, install } = require('./stepper');
const { service } = require('./http-service');
const { logger } = require('./logger');
const chalk = require('chalk');
const { NodeSSH } = require('node-ssh');
const os = require('os');

program.command('run [projects...]')
  .option('-m, --maintenance', 'Stop and restart Nginx or httpd systemctl services after duplications')
  .option('-s, --ssh <url>', "Target ssh url to execute rgds's file duplication")
  .option('-p, --parrallel', 'Enable parralel duplciations of rgds')
  .addOption(new program.Option('-o, --only <mode>', 'Duplicate only database or files of rgds').choices(['db', 'file', 'all']).default('all'))
  .action(async (projects, cmd) => {
    if (cmd.maintenance && service.canUseService() && service.isActive()) service.stop();
    await run(projects, cmd);
    if (cmd.maintenance && service.canUseService()) service.start();
    logger.shutdown(() => {
      process.exit(0);
    });
  });

program.command('configure')
  .action(async () => {
    await configure();
    process.exit(0);
  });

program.command('install')
  .action(async () => {
    await install();
    process.exit(0);
  });

program.command('ssh-test <url> [keyPath]')
  .action(async (url, keyPath = `${os.homedir()}/.ssh/id_rsa`) => {
      let ssh = new URL(url);
      let sshConnection = new NodeSSH();
      await sshConnection.connect({
        host: ssh.hostname,
        username: ssh.username,
        password: ssh.password,
        privateKey: !ssh.password ? keyPath : undefined,
        port: ssh.port,
        readyTimeout: 60000,
        debug: (log) => {
          if (process.env.DEBUG) {
            console.debug(log);
          }
        }
      });

      try {
        const test = await sshConnection.execCommand(`azcopy --help`, { execOptions: { pty: true }, });
        
        await sshConnection.dispose();
        if (test.stderr) {
          console.error(chalk.red(test.stderr));
        } else {
          console.log(chalk.green(test.stdout));
        }
      } catch(err) {
        console.error(err);
      }
    });

program.parse();