const conf = require('../configuration.json');
const { v4 } = require('uuid');
const os = require('os');

exports.source = {
  database: {
    defaultDb: process.env.MYSQL_SOURCE_DEFAULT_DB || conf.source.database.defaultDb,

    mysqlHost: process.env.MYSQL_SOURCE_HOST || conf.source.database.host,
    mysqlUser: process.env.MYSQL_SOURCE_USER || conf.source.database.user,
    mysqlPassword: process.env.MYSQL_SOURCE_PASSWORD || conf.source.database.password,
    mysqlPort: process.env.MYSQL_SOURCE_PORT || conf.source.database.port,
    mysqlSSL: ( process.env.MYSQL_SOURCE_SSL  || conf.source.database.ssl ) === "true",
  },
  file : {
    azureUrl: process.env.AZURE_SOURCE_STORAGE_URL || conf.source.file.azure.url,
    azureToken: process.env.AZURE_SOURCE_TOKEN || conf.source.file.azure.token,
    basePath: process.env.SOURCE_BASE_PATH || conf.source.file.basePath,
  }
}

exports.target = {
  database: {
    defaultDb: process.env.MYSQL_TARGET_DEFAULT_DB || conf.target.database.defaultDb,

    mysqlHost: process.env.MYSQL_TARGET_HOST || conf.target.database.host,
    mysqlUser: process.env.MYSQL_TARGET_USER || conf.target.database.user,
    mysqlPassword: process.env.MYSQL_TARGET_PASSWORD || conf.target.database.password,
    mysqlPort: process.env.MYSQL_TARGET_PORT || conf.target.database.port,
    mysqlSSL: ( process.env.MYSQL_TARGET_SSL  || conf.target.database.ssl ) === "true",
  },
  file : {
    azureUrl: process.env.AZURE_TARGET_STORAGE_URL || conf.target.file.azure.url,
    azureToken: process.env.AZURE_TARGET_TOKEN || conf.target.file.azure.token,
    
    basePath: process.env.TARGET_BASE_PATH || conf.target.file.basePath,
  }
}

exports.azureLogLevel = process.env.AZURE_LOG_LEVEL  || conf.azureLog || "ERROR";
exports.azCopyPath = process.env.AZ_COPY_PATH || conf.azCopyPath || 'azcopy';

exports.ignoreTables = conf.ignoreTables || [];

exports.sshPrivateKeyPath = process.env.SSH_PRIV_KEY_PATH || `${os.homedir()}/.ssh/id_rsa`;

exports.port = process.env.PORT || 3000;
exports.apiKey = process.env.API_KEY || v4();

exports.noHtml = process.env.NO_UI === 'true';