const mysql = require('mysql');

class MysqlConnector {
  constructor() {
    this.connected = false;
    this.options = {};
  }

  createConnection(connectionOptions) {
    if (!connectionOptions.host || !connectionOptions.user || !connectionOptions.password) throw new Error(`No configuration found for mysql host, user or password`);
    this.options = connectionOptions;
    return connectionOptions.ssl ? mysql.createPool(connectionOptions) : mysql.createConnection(connectionOptions);
  }


  async connect(connectionOptions) {
    if (!this.connected) {
      this.connection = this.createConnection(connectionOptions);
      return new Promise((resolve, reject) => {

        if (!this.options.ssl)  {
          this.connection.connect((err) => {
            if (err) return reject(err);
            this.connected = true;
            resolve();
          });
        } else {
          this.connected = true;
          resolve();
        }
       
      });
    }
  }

  async end() {
    if (this.connected) {
      return new Promise((resolve, reject) => {
        if (!this.options.ssl) {
          this.connection.end((err) => {
            if (err) return reject(err);
            this.connected = false;
            resolve();
          });
        } else {
          this.connected = false;
          resolve();
        }
      });
    }
    
  }

  async query( sql, args ) {
    if (this.connected) {
      return new Promise( ( resolve, reject ) => {
        this.connection.query( sql, args, ( err, rows ) => {
            if ( err )
                return reject( err );
            resolve( rows );
        } );
      });
    }
  }
}

exports.db = new MysqlConnector();