const { db } = require('../mysql-connector');
const Importer = require('mysql-import');
const { writeFileSync } = require('fs');
const { spawnSync } = require('child_process');
const chalk = require('chalk');


const { source, target } = require('../conf');

exports.run = async (project, logger) => {
  if (logger) logger.info('Duplicate database', 'begining');
  console.log(chalk.blue('Duplicate database'), 'begining');
  let result = true;

  let options = {};

  if (logger) logger.info('Duplicate databse - 1', 'dump project\'s database to sql file');
  console.log(chalk.blue('Duplicate databse - 1'), 'dump project\'s database to sql file');
  const sqlDumpPath = `./${project}.sql`;

  const dump = spawnSync('mysqldump', [
    '-h',
    source.database.mysqlHost,
    '-u',
    source.database.mysqlUser,
    `-p${source.database.mysqlPassword}`,
    ...(ignoreTables.map(table => `--ignore-table=${project}.${table}`)),
    '--ssl',
    project,
  ]);

  if (dump && dump.stderr && dump.stderr.toString()) {
    throw new Error(dump.stderr.toString());
  } else {
    writeFileSync(sqlDumpPath, dump.stdout.toString());
  }

  if (logger) logger.info('Duplicate database - 2', 'create or recreate database');
  console.log(chalk.blue('Duplicate database - 2'), 'create or recreate database');
  options = {
    multipleStatements: true,
    host: target.database.mysqlHost,
    user: target.database.mysqlUser,
    password: target.database.mysqlPassword,
    port: target.database.mysqlPort || 3306,
    ssl: target.database.mysqlSSL
  };
  await db.connect(options);
  try {
    result = await db.query(`drop database if exists ${project}; create database ${project};`);
  } catch (err) {
    await db.end();
    throw err;
  }
  
  await db.end();
    

  if (logger) logger.info('Duplicate database - 3', 'execute project\' sql file');
  console.log(chalk.blue('Duplicate database - 3'), 'execute project\' sql file');
  
  let importOptions = {
    host: target.database.mysqlHost,
    user:  target.database.mysqlUser,
    password:  target.database.mysqlPassword,
    port: target.database.mysqlPort,
    database: project,
    ssl : target.database.mysqlSSL
  };
  
  let importer;
  try {
    importer = new Importer(importOptions);  
    importer.onProgress(progress=>{
      var percent = Math.floor(progress.bytes_processed / progress.total_bytes * 10000) / 100;
      if (logger) logger.info(`${percent}% Completed`);
      console.log(`${percent}% Completed`);
    });
    await importer.import(sqlDumpPath);
  } catch (err) {
    result = undefined;
    throw err;
  } finally {
    if (importer) importer.disconnect();
  }


  if (logger) logger.info('Duplicate databse - 4', 'retrieve project\'s globalliste and organisations_projects tuples');
  console.log(chalk.blue('Duplicate databse - 4'), 'retrieve project\'s globalliste and organisations_projects tuples');
  options = {
    multipleStatements: true,
    host: source.database.mysqlHost,
    user: source.database.mysqlUser,
    password: source.database.mysqlPassword,
    port: source.database.mysqlPort || 3306,
    database: source.database.defaultDb,
    ssl: source.database.mysqlSSL,
    dateStrings: true
  };
  let projectTuple;
  // let organisations;
  try {
    await db.connect(options);
    const projects = await db.query(`SELECT * FROM globalliste WHERE g_nom="${project}";`);
    // organisations = await db.query(`SELECT * FROM organisations_projects WHERE g_id="${projects[0].g_id}";`);
    projectTuple = projects[0];
  } catch (err) {
    result = false;
    await db.end();
    throw err;
  }

  await db.end();
  if (logger) logger.info('Duplicate databse - 5', 'insert project\'s globalliste and organisations_projects tuples');
  console.log(chalk.blue('Duplicate databse - 5'), 'insert project\'s globalliste and organisations_projects tuples');
  options = {
    multipleStatements: true,
    host: target.database.mysqlHost,
    user: target.database.mysqlUser,
    password: target.database.mysqlPassword,
    port: target.database.mysqlPort || 3306,
    database: target.database.defaultDb,
    ssl: target.database.mysqlSSL
  };
  try {
    
    await db.connect(options);
    result = await db.query(`INSERT INTO globalliste (\`g_nom\`, \`g_desc\`, \`hash2\`, \`g_internalname\`, \`g_createdate\`, \`g_basepath\`, \`g_photo\`, \`g_xml\`, \`templateMail\`, \`lastactivity\`, \`client\`, \`cle\`, \`serial\`, \`linkemail\`, \`server_time\`, \`server_update_time\`, \`g_c\`, \`g_infos\`, \`g_serverinfos\`, \`g_maintenance\`, \`g_running\`, \`sxd_creationdate\`, \`sxd_typeClient\`, \`sxd_bigData\`, \`entreprise\`, \`ext_token\`, \`g_logo_file_entete\`, \`centralbase\`, \`customer_id\`, \`timezone\`) 
                                              VALUES ('${projectTuple.g_nom}', '${projectTuple.g_desc}', '${projectTuple.hash2}', '${projectTuple.g_internalname}', '${projectTuple.g_createdate}', '${projectTuple.g_basepath}', '${projectTuple.g_photo}', '${projectTuple.g_xml}', '${projectTuple.templateMail}', '${projectTuple.lastactivity}', ${projectTuple.client}, ${projectTuple.cle}, ${projectTuple.serial}, ${projectTuple.linkemail}, ${projectTuple.server_time}, ${projectTuple.server_update_time}, ${projectTuple.g_c}, ${projectTuple.g_infos}, ${projectTuple.g_serverinfos}, ${projectTuple.g_maintenance}, ${projectTuple.g_running}, '${projectTuple.sxd_creationdate}','${projectTuple.sxd_typeClient}', ${projectTuple.sxd_bigData}, '${projectTuple.entreprise}', ${projectTuple.ext_token}, ${projectTuple.g_logo_file_entete}, '${projectTuple.centralbase}', ${projectTuple.customer_id}, '${projectTuple.timezone}');`);
    if (!result) {
      throw new Error('Impossible to insert globaliste project\'s tuple');
    }

    /*
    TODO : need to define rules of organisations project association
    if we insert id as found, targeted environment will not have the same organisations ids
    if (logger) logger.info(result);
    for (const organisation of organisations) {
      result = await db.query(`INSERT INTO organisations_projects (organisation_id, g_id) VALUES ('${organisation.organisation_id}', '${result.insertId}')`);
    }
       if (!result) {
      throw new Error('Impossible to insert organisations_projects project\'s tuples');
    }
    */

  } catch (err) {
    result = false;
    await db.end();
    throw err;
  }

  await db.end();
  




  if (logger) logger.info('Duplicate database', 'end');
  console.log(chalk.blue('Duplicate database'), 'end');
  return result;
};

exports.verify = async (project, logger) => {
  if (logger) logger.info('Verify database', 'begining');
  console.log(chalk.yellow('Verify database'), 'begining');
  let result = true;
  let options = {
    multipleStatements: true,
    host: source.database.mysqlHost,
    user: source.database.mysqlUser,
    password: source.database.mysqlPassword,
    port: source.database.mysqlPort || 3306,
    database: project,
    ssl: source.database.mysqlSSL
  };

  try {
    await db.connect(options);
    result = true;
  } catch (err) {
    result = false;
    if (logger) logger.error('Verify database', 'rgds source database unreachable');
    console.error(chalk.red('Verify database'), 'rgds source database unreachable');
    await db.end();
  }
  await db.end();

  if (result) {
    options = {
      multipleStatements: true,
      host: source.database.mysqlHost,
      user: source.database.mysqlUser,
      password: source.database.mysqlPassword,
      port: source.database.mysqlPort || 3306,
      database: source.database.defaultDb,
      ssl: source.database.mysqlSSL
    };

    try {
      await db.connect(options);
      const projects = await db.query(`SELECT g_id FROM globalliste WHERE g_nom="${project}";`);
      if (projects && projects.length && projects[0].g_id) {
        /*
        const organisations = await db.query(`SELECT organisation_id FROM organisations_projects WHERE g_id="${projects[0].g_id}";`);
        if (organisations && organisations.length) {
          result = true;
        } else {
          result = false;
        }
        */
        result = true;
      } else {
        if (logger) logger.error('Verify database', 'rgds not in globalliste');
        console.error(chalk.red('Verify database'), 'rgds not in globalliste');
        result = false;
      }
    } catch (err) {
      result = false;
      await db.end();
    }

    await db.end();
  }

  
  if (result) {
    options = {
      multipleStatements: true,
      host: target.database.mysqlHost,
      user: target.database.mysqlUser,
      password: target.database.mysqlPassword,
      port: target.database.mysqlPort || 3306,
      database: target.database.defaultDb,
      ssl: target.database.mysqlSSL
    };
    
    try {
      await db.connect(options);
      result = true;
    } catch (err) {
      if (logger) logger.error('Verify database', 'target database unreachable');
      console.error(chalk.red('Verify database'), 'target database unreachable', err);
      result = false;
      await db.end();
    }
    await db.end();
  }
  


  if (logger) logger.info('Verify database', 'end');
  console.log(chalk.yellow('Verify database'), 'end');
  return result;
};