
const {
  source,
  target,
  azureLogLevel,
  sshPrivateKeyPath,
  azCopyPath
} = require('../conf');

const { spawnSync } = require('child_process');
const chalk = require('chalk');
const {
  existsSync,
  appendFileSync,
} = require('fs');
const { NodeSSH } = require('node-ssh');


exports.run = async (project, logger, sshUrl) => {
  if (logger) logger.info('Duplicate files', 'begining');
  console.log(chalk.blue('Duplicate files'), 'begining');
  let result;



  if (logger) logger.info('Duplicate files', 'executing azcopy');
  console.log(chalk.blue('Duplicate files'), 'executing azcopy');
  try {
    const sourceFolders = source.file.basePath ? `${source.file.basePath}/${project}` : `${source.file.azureUrl}/${project}${source.file.azureToken}`;
    const targetFolders = target.file.basePath ? `${target.file.basePath}/${project}` : `${target.file.azureUrl}${target.file.azureToken}`;
    let copy;
    if (sshUrl) {
      let ssh = new URL(sshUrl);
      let sshConnection = new NodeSSH();
      await sshConnection.connect({
        host: ssh.hostname,
        username: ssh.username,
        password: ssh.password,
        privateKey: !ssh.password ? sshPrivateKeyPath : undefined,
        port: ssh.port,
        readyTimeout: 60000,
        debug: (log) => {
          if (process.env.DEBUG) {
            console.debug(log);
          }
        }
      });

      copy = await sshConnection.execCommand(`${azCopyPath} copy ${sourceFolders} "${targetFolders}" --recursive=true --log-level=${azureLogLevel}`, { execOptions: { pty: true } }); 
      await sshConnection.dispose();
    } else {
      copy = spawnSync(azCopyPath, ['copy', `${sourceFolders}`, `${targetFolders}`, `--recursive=true`, `--log-level=${azureLogLevel}`]);
    }
    if (copy.stderr && copy.stderr.toString() || copy.error) {
      throw copy.stderr ? new Error(copy.stderr.toString()) : copy.error;
    } else if (copy.stdout.toString().includes('RESPONSE ERROR')) {
      throw new Error(copy.stdout.toString());
    } else {
      if (logger) logger.info(copy.stdout.toString());
      console.log(copy.stdout.toString());
      result = true;
    }

  } catch (err) {
    result = false;
    throw err;
  }


  /*
  if (target && target.url) {
    if (logger) logger.info('Duplicate files', 'Edit mailer script'); 
    try {
      appendFileSync('/home/scripts/7pm_launchrapide.sh', `/usr/bin/php /home/datareso/www/rgoc12.1/services/rgoc_modele/launch.php ${project}\n`);
      result = true;
    } catch (err) {
      result = false;
      throw err;
    }
    
  }
  */

  if (logger) logger.info('Duplicate files', 'end');
  console.log(chalk.blue('Duplicate files'), 'end');
  return result;
};

exports.verify = async (project, logger, sshUrl) => {
  if (logger) logger.info('Verify files', 'begining');
  console.log(chalk.yellow('Verify files'), 'begining');
  let result = true;

  try {
    if (source.file.basePath && !sshUrl) {
      result = existsSync(`${source.file.basePath}/${project}`);
    } else if (source.file.basePath && sshUrl) {
      const ssh = new URL(sshUrl);
      const sshConnection = new NodeSSH();
      await sshConnection.connect({
        host: ssh.hostname,
        username: ssh.username,
        password: ssh.password,
        privateKey: !ssh.password ? sshPrivateKeyPath : undefined,
        port: ssh.port,
        readyTimeout: 60000,
        debug: (log) => {
          if (process.env.DEBUG) {
            console.debug(log);
          }
        }
      });

      const ls = await sshConnection.execCommand(
        `ls ${source.file.basePath}/${project}`
      );

      if (ls.stderr.toString()) {
        result = false;
      }

      await sshConnection.dispose();

    } else if (source.file.azureUrl && source.file.azureToken) {
      const copy = spawnSync(azCopyPath, ['list', `${source.file.azureUrl}/${project}${source.file.azureToken}`]);
      if (copy.stderr && copy.stderr.toString() || copy.error) {
        throw copy.stderr ? new Error(copy.stderr.toString()) : copy.error;
      } else if (copy.stdout.toString().includes('RESPONSE Status: 404')) {
        throw new Error(copy.stdout.toString());
      } else {
        result = true;
      }
    } else {
      result = false;
    }
  } catch (err) {
    console.log(err);
    result = false;
  }


  if (logger) logger.info('Verify files', 'end');
  console.log(chalk.yellow('Verify files'), 'end');
  return result;
};